package com.iflytek.cloud;

import android.os.Environment;

import java.util.Locale;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.resource.Resource;

/**
 * <h3>设置</h3>
 * <p>
 * 		设置是否显示控制台日志，是否保存共享库调试日志等。
 * </p>
 * <p>
 * 		SDK会打印两种日志：Msc.jar的控制台日志，及libmsc.so的共享库日志。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 *
 */
public class Setting {
	
	/**
	 * <h4>日志级别</h4>
	 * <p>日志按重要程度区分了级别。</p>
	 * @see Setting#setLogLevel
	 */
	public enum LOG_LEVEL
	{
		/** 保留所有日志  */
		all,   
		/** 保留INFO等级日志 */
		detail,
		/** 保留DEBUG等级日志 */
		normal,  
		/** 保留ERR等级日志 */
		low,   
		/** 不保留任何日志*/
		none,    
	}
	
	/** <h4>共享库日志默认路径</h4> */
	final public static String DEFAULT_LOG_PATH = Environment.getExternalStorageDirectory().getPath()+"/msc/msc.log";
	/** <h4>是否打印日志（控制台和共享库）默认值</h4> */
	final public static boolean DEFAULT_IS_SHOW_LOG = true;
	/** <h4>是否保存性能日志文件默认值</h4> */
	final public static boolean DEFAULT_IS_SAVE_TEST_LOG = false;
	/** <h4>是否允许获取位置信息默认值</h4> */
	final public static boolean DEFAULT_IS_LOCATION_ENABLE = true;

	// 日志路径。 默认值 DEFAULT_LOG_PATH 
	private static String    sLogPath = DEFAULT_LOG_PATH;

	// 是否保存性能日志文件
	private static boolean sIsSaveTestLog   = DEFAULT_IS_SAVE_TEST_LOG;
	// 是否允许获取经纬度信息
	private static boolean sIsLocationEnable = DEFAULT_IS_LOCATION_ENABLE;
	
	private Setting() {
		// do nothing
	}
	
	/**
	 * <h4>设置日志级别</h4>
	 * <p>
	 * 		日志级别，默认值为{@link LOG_LEVEL#normal}。应用层根据调试的需要，
	 * 可以设置一个自己需要的级别，或使用默认的值。在应用发布时，为了减少日志的开销，又不影响问题的调试，
	 * 建议设置为 {@link LOG_LEVEL#normal} 级别。
	 * </p>
	 * <p>共享库的日志级别，只有在共享库日志保存路径设置有效时，才会生效。</p>
	 * <p>
	 * 		jar的android logcat根据日志级别打印：当级别为{@link LOG_LEVEL#all}或{@link LOG_LEVEL#detail}时，
	 * 打印所有日志；当级别为{@link LOG_LEVEL#normal}时打印会话开始、结束，错误等关键日志，当级别为
	 * {@link LOG_LEVEL#low} 时，仅打印错误日志；当级别为
	 *	{@link LOG_LEVEL#none}时，关闭所有日志打印。
	 * </p>
	 * @param level 日志级别
	 * @see LOG_LEVEL
	 * @see #getLogLevel
	 * @see #setLogPath
	 */
	public static void setLogLevel( final LOG_LEVEL level )
	{
		try{
			if( null != level ){
				DebugLog.setLogLevel( DebugLog.LOG_LEVEL.values()[level.ordinal()] );
			}
		}catch( Exception e ){
			DebugLog.LogE( e );
		}// try-catch
	}
	
	/**
	 * <h4>获取日志级别</h4>
	 * @return 日志级别，默认值为{@link LOG_LEVEL#normal}
	 * @see #setLogLevel
	 */
	public static LOG_LEVEL getLogLevel(){
		LOG_LEVEL level = LOG_LEVEL.none;
		
		try{
			level = LOG_LEVEL.values()[ DebugLog.getLogLevel().ordinal() ];
		}catch( Exception e ){
			DebugLog.LogE( e );
		}// try-catch
		
		return level;
	}
	
	/**
	 * <h4>设置共享库日志保存路径</h4>
	 * <p>
	 * 		共享库日志路径，默认值为{@link #DEFAULT_LOG_PATH }，即当前应用运行路径。
	 * 在应用层设置的路径有效时，共享库日志才会保存。
	 * </p>
	 * @param path 共享库日志路径值，含文件名。
	 * @see #setLogLevel
	 * @see #getLogPath
	 */
	public static void setLogPath( final String path ){
		sLogPath = path;
	}
	
	/**
	 * <h4>获取共享库日志保存路径</h4>
	 * @return 获取共享库日志保存路径
	 * @see #setLogPath
	 */
	public static String getLogPath(){
		return sLogPath;
	}
	
	/**
	 * <h4>设置是否打印日志信息</h4>
	 * <p>
	 * 		设置是否打印logcat日志和共享库日志, 默认值为{@link #DEFAULT_IS_SHOW_LOG}。
	 * </p>
	 * 
	 * @param isShowLog false->不显示，true->显示
	 * @see #getShowLog
	 */
	public static void setShowLog( final boolean isShowLog)
	{
		DebugLog.setShowLog( isShowLog );
	}
	
	/**
	 * <h4>获取是否打印日志信息</h4>
	 * @return 设置是否打印logcat日志和共享库日志。
	 * @see #setShowLog
	 */
	public static boolean getShowLog(){
		return DebugLog.getShowLog();
	}
	
	/**
	 * <h4>设置是否保存性能日志</h4>
	 * <p>
	 * 		性能日志有助于SDK开发人员了解SDK的性能情况，应用层可忽略此日志。 
	 * </p>
	 * @param isSaveTestLog 是否保存性能日志，默认值{@link #DEFAULT_IS_SAVE_TEST_LOG}
	 * @see #getSaveTestLog
	 */
	public static void setSaveTestLog( final boolean isSaveTestLog ){
		sIsSaveTestLog = isSaveTestLog;
	}
	
	/**
	 * <h4>获取是否保存性能日志</h4>
	 * @return 是否保存性能日志
	 * @see #setSaveTestLog
	 */
	public static boolean getSaveTestLog(){
		return sIsSaveTestLog;
	}
	
	/**
	 * <h4>设置是否允许获取位置信息</h4>
	 * 
	 * @param enable 是否允许
	 * @see #setLocationEnable
	 */
	public static void setLocationEnable(boolean enable){
		sIsLocationEnable = enable;
	}
	
	/**
	 * <h4>获取是否允许获取位置信息</h4>
	 * @return 是：true，否：false
	 * @see #getLocationEnable
	 */
	public static boolean getLocationEnable(){
		return sIsLocationEnable;
	}

	/**
	 * <h4>设置当前SDK的地区</h4>
	 * <p>
	 * 		设置当前应用要指定SDK适应的地区，使 SDK 的在返回必要的信息（如错误提示）时，
	 * 适应指定地区的语言。
	 * </p>
	 * <p>
	 * 		此函数与 {@link com.iflytek.cloud.ui.RecognizerDialog#setUILanguage(Locale)} 的作用一致。
	 * </p>
	 * @param locale 地区值，当前支持{@link Locale#CHINA}, {@link Locale#US}, 
	 * 		{@link Locale#TRADITIONAL_CHINESE}
	 * @see com.iflytek.cloud.ui.RecognizerDialog#setUILanguage(Locale)
	 */
	public static void setLocale( final Locale locale ){
		Resource.setUILanguage( locale );
	}
}
