package com.iflytek.cloud;

import android.os.Bundle;



/**
 * <h3>声纹监听器</h3>
 * <p>通过实现此接口，获取当前声纹的状态和结果</p>
 * <p>识别监听器，请参考{@link RecognizerListener}</p>
 * <p>合成监听器，请参考{@link SynthesizerListener}</p>
 * <p>语音语义监听器，请参考{@link SpeechUnderstanderListener}</p>
 * <p>文本语义监听器，请参考{@link TextUnderstanderListener}</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see SpeakerVerifier#startListening(VerifierListener)
 */
public interface VerifierListener {
	/**
	 * <h4>音量变化</h4>
	 * <p>
	 *     当开始识别，到停止录音（停止写入音频流）或SDK返回最后一个结果自动结束识别为止，
	 * SDK检测到音频数据（正在录音或写入音频流）的音量变化时，多次通过此函数回调，告知应
	 * 用层当前的音量值。应用层可通过此函数传入的值变化，改变自定义UI的画面等。
	 * </p> 
	 * @param volume 当前音量值，范围[0-30]
	 * @see SpeakerVerifier#startListening
	 * @see #onBeginOfSpeech
	 * @see #onEndOfSpeech
	 * @see SpeakerVerifier#stopListening
	 */
	void onVolumeChanged(int volume , byte[] data);

	/**
	 * <h4>开始说话</h4>
	 * <p>
	 * 		在录音模式({@link SpeechConstant#AUDIO_SOURCE 音频源}参数设为 > -1时 )下，
	 * 调用{@link SpeakerVerifier#startListening 开始录音函数}后，会自动开启系统的录音
	 * 机，并在录音机开启后，会回调此函数（这中间的过程应该在几毫秒内，可以忽略，除非系
	 * 统响应很慢）。
	 * </p>
	 * <p>
	 * 		应用层可通过此函数回调，告知用户，当前可开始说话。若应用层使用的是声音提示，
	 * 则应该在调用{@link SpeakerVerifier#startListening 开始录音函数}前，提放提示音，
	 * 并在提示音非静音数据播放结束时，调用
	 * {@link SpeakerVerifier#startListening 开始录音函数}，开始录音。若太早调用，则
	 * 可能会把提示音录进要识别的音频中，若太晚，则可能会漏掉部分用户说话的音频。
	 * </p>
	 * @see SpeakerVerifier#startListening
	 * @see #onEndOfSpeech
	 */
	void onBeginOfSpeech();
	
	/**
	 * <h4>结束说话</h4>
	 * <p>
	 * 		在SDK检测到音频的静音端点时，回调此函数（在录音模式或写音频模式下都会回调，
	 * 应用层主动调用{@link SpeakerVerifier#stopListening}则不会回调此函数，
	 * 在识别出错时，可能不会回调此函数）。在此函数回调后，当前识别会话可能并没有结束，
	 * 识别结果可能还要等待一定的时间才会返回。
	 * </p>
	 * <p>
	 * 		此函数回调后，应用层应立即停止调用{@link SpeakerVerifier#writeAudio}写入音
	 * 频数据，(当音频源设置为音频流时（{@link SpeechConstant#AUDIO_SOURCE}为-1时)
	 * 否则，再通过{@link SpeakerVerifier#writeAudio}写入的音频也会被忽略。
	 * </p>
	 * <p>
	 * 		应用层可以通过此函数回调，告知用户，当次说话已结束，正在等待识别结果（若结果
	 * 未返回）等。
	 * </p>
	 * 
	 * @see #onBeginOfSpeech
	 * @see #onResult
	 * @see #onError
	 * @see SpeakerVerifier#writeAudio
	 * @see SpeakerVerifier#stopListening
	 */
	void onEndOfSpeech();
	
	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。
	 * </p>
	 * <p>
	 * 		声纹注册：<br>
	 * 在声纹注册过程，此函数可能会回调多次：<br>
	 * 1，自由说模式下，如果不出现错误，则回调一次，返回注册成功的结果；如果出现错误
	 * 而引起重试，则根据重试次数，返回多次。<br>
	 * 2，随机密码和固定文本模式下，根据注册次数，回调多次，如果有出现错误而引起重试，
	 * 回调次数则相应增加。
	 * </p>
	 * <p>
	 * 		在注册过程中，说完当次的密码后，等待此函数回调当次注册是否正确，才能继续说下
	 * 一个密码，此时，会话还在继续，且录音机依然打开，所以应该在一个相对安静的环境下
	 * 进行注册，以减少环境的噪音对结果的影响。在注册过程，如果{@link VerifierResult#ret}
	 * 返回成功，而{@link VerifierResult#err}有错误，则当前会话可以继续，重试当次注册就可
	 * 以。参考{@link VerifierResult}对注册结果的说明。
	 * </p>
	 * <p>
	 * 		声纹验证：<br>
	 * 在声纹验证过程，一次会话回调一次，告知应用验证结果，验证通过或失败。
	 * </p>
	 * <p>
	 * 		当最后一个结果返回时，本次会话结束，录音也会停止，在重新调用
	 *  {@link SpeakerVerifier#startListening}开启新的识别会话前，
	 * 停止调用{@link SpeakerVerifier#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * 当出现错误，或应用层调用{@link SpeakerVerifier#cancel}取消当次识别时，在当次识
	 * 别会话过程可能不会回调此函数。
	 * </p>
	 * 
	 * @param result 结果数据
	 * 
	 * @see #onError
	 * @see SpeakerVerifier#cancel
	 * @see VerifierResult
	 */
	void onResult(VerifierResult result);
	
	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，录音也会停止。应在再次调
	 * 用{@link SpeakerVerifier#startListening}开启新的识别会话前，
	 * 停止调用{@link SpeakerVerifier#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * </p>
	 * @param error 错误类型
	 * @see SpeechError
	 * @see #onResult
	 * @see SpeakerVerifier#startListening
	 */
	void onError(SpeechError error);
	
    /**
     * <h4>事件</h4>
     * <p>
     * 		扩展用接口，由具体业务进行约定。例如eventType为0显示网络状态，agr1为网络连接值。
     * </p>
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 消息内容
     * 
     * @see SpeechEvent
     */
    void onEvent(int eventType, int arg1,int arg2, Bundle obj);
}
