package com.iflytek.cloud.msc.isr;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.GrammarListener;
import com.iflytek.cloud.LexiconListener;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class IsrSession extends MscSession {

    private final static int       ISR_AUDIO_SAMPLE_CONTINUE = 0x02;
    private final static int       ISR_AUDIO_SAMPLE_LAST     = 0x04;

    /** member properties */
    private MSCSessionInfo         mSessionOut               = new MSCSessionInfo();
    private MSCSessionInfo         mWriteout                 = new MSCSessionInfo();
    // 通过QISRGetResult获取到的识别结果
    private byte[]                 mResult                   = null;

    private static GrammarListener mGrammarListener;
    private static LexiconListener mLexiconListener;

    //用户词表名称
    private String mLexiconName ="";
    
    
    @Override
    public int sessionBegin(final Context context, final String grammar,
            final MscHandler handler) throws SpeechError,
            UnsupportedEncodingException {
        String params = ParamBuilder.composeIsrSessionParam(context, grammar,
                handler);
        long d1 = SystemClock.elapsedRealtime();
        //加锁防止外部多线程造成同时ssb成功，之后回调造成的崩溃。
        DebugLog.LogD( "QISRSessionBegin begin, grammar: "+grammar );
        synchronized (IsrSession.class) {
    		PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
        	if (TextUtils.isEmpty(grammar)) {
        		mClientID = MSC.QISRSessionBegin(null,
        				params.getBytes(handler.getParamEncoding()), mSessionOut);
        	} else {
        		mClientID = MSC.QISRSessionBegin(
        				grammar.getBytes(handler.getParamEncoding()),
        				params.getBytes(handler.getParamEncoding()), mSessionOut);
        	}
    		PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
        }
        DebugLog.LogD("QISRSessionBegin end: " + mSessionOut.errorcode
                + " time:" + (SystemClock.elapsedRealtime() - d1));

        int errorCode = mSessionOut.errorcode;
        if (0 != errorCode) {
            // 外面做超时处理，防止10129直接返回
            if (errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
                    && errorCode != ErrorCode.MSP_ERROR_OVERFLOW
                    && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION) {
                throw new SpeechError(errorCode);
            }
        }
        return errorCode;
    }

    @Override
    public void sessionEnd(final String reason) {
        if (null == mClientID) {
            return;
        }
        DebugLog.LogD("sessionEnd enter ");
        long d1 = System.currentTimeMillis();
        boolean bRet = 0 == MSC.QISRSessionEnd(mClientID, reason.getBytes());
        DebugLog.LogD("sessionEnd leavel:" + bRet + " time:"
                + (System.currentTimeMillis() - d1));
        mClientID = null;
        mSessionID = null;
    }

    synchronized public void pushAudioData(final byte[] waveData,
            final int waveLen) throws SpeechError {
        writeData(waveData, waveLen, ISR_AUDIO_SAMPLE_CONTINUE);
    }

    synchronized public void pushEndFlag() throws SpeechError {
        byte[] data = new byte[0];
    	PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
    	DebugLog.LogD( "IsrSession pushEndFlag" );
        writeData(data, 0, ISR_AUDIO_SAMPLE_LAST);
    }

    /**
     * upload data to server
     * 
     * @param waveData 写入的wav 数据
     * @param waveLen 数据长度
     * @param audioStatus 音频状态
     * @throws SpeechError MSC error类型
     */

    synchronized private void writeData(final byte[] waveData,
            final int waveLen, final int audioStatus) throws SpeechError {
        int nRet = MSC.QISRAudioWrite(mClientID, waveData, waveLen,
                audioStatus, mWriteout);

        mSessionOut.sesstatus = mWriteout.sesstatus;
        DebugLog.LogI("QISRAudioWrite length:" + waveLen);
        if (0 != nRet) {
            throw new SpeechError(mWriteout.errorcode);
        }
    }

    /**
     * 
     * @return true if find record end point
     */
    synchronized public int getEpStatus() {
        int eps = mWriteout.epstatues;
        return eps;
    }

    /**
     * get audio data volume value
     * 
     * @return
     */
    synchronized public int getAudioVolume() {
        int ret = 0;
        int vadret = 0;
        try {
            vadret = MSC
                    .QISRGetParam(mClientID, "volume".getBytes(), mWriteout);
            if (vadret == 0) {
                String volume = new String(mWriteout.buffer);
                // MscLog.Log(TAG,"VAD CHECK RET = " + vadret + " vad=" +
                // mWriteout.getQisrParamValue().length + new
                // String(mWriteout.getQisrParamValue()));
                ret = Integer.parseInt(new String(volume));
            } else {
                DebugLog.LogI("VAD CHECK FALSE");
            }
        } catch (Exception e) {
            DebugLog.LogI("getAudioVolume Exception vadret = " + vadret);
        }
        return ret;
    }

    /**
     * get int value
     * 
     * @return
     */
    synchronized public int getIntValue(final String tag) {
        if (mClientID == null) {
            return 0;
        }
        int ret = 0;
        try {
            String value = getStringValue(tag);
            if (!TextUtils.isEmpty(value)) {
                ret = Integer.parseInt(new String(value));
            } else {
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
        return ret;
    }

    /**
     * 设置会话参数，如rec_start、reco_ustop信息
     * 
     * @return
     */
    synchronized public boolean setParam(final String key, final String value) {
        if (TextUtils.isEmpty(key) || TextUtils.isEmpty(value)
                || mClientID == null) {
            return false;
        }
        int ret = -1;
        try {
            ret = MSC.QISRSetParam(mClientID, key.getBytes(DataUtil.UTF8),
                    value.getBytes(DataUtil.UTF8));
        } catch (UnsupportedEncodingException e) {
            DebugLog.LogE(e);
        }
        return (ret == 0);
    }

    /**
     * get String value
     * 
     * @return
     */
    synchronized public String getStringValue(final String tag) {
        if (mClientID == null) {
            return null;
        }
        int code = 0;
        try {
            code = MSC.QISRGetParam(mClientID, tag.getBytes(), mSessionOut);
            if (code == 0) {
                return new String(mSessionOut.buffer);
            } else {
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * buildGrammar
     * 
     * @return
     */
    public int buildGrammar(final String grammarType,
    		final String grammarContent, final GrammarListener grammarListener,
    		final HashParam mSessionParams) {
    	mGrammarListener = grammarListener;
    	String params = mSessionParams.toString();
    	
    	//文本编码格式
    	String textEncoding = mSessionParams.getString(SpeechConstant.TEXT_ENCODING, DataUtil.UTF8);
    	//参数编码格式
    	String paramEncoding = mSessionParams.getString(MscKeys.KEY_PTE, DataUtil.UTF8);
    	
    	int ret = ErrorCode.MSP_ERROR_FAIL;
    	try {
    		byte[] tempGrammarContent = grammarContent.getBytes(textEncoding);
    		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
    		DebugLog.LogD( "QISRBuildGrammar begin, garmmar: "+grammarContent );
    		ret = MSC.QISRBuildGrammar(grammarType.getBytes(paramEncoding),
    				tempGrammarContent,
    				tempGrammarContent.length,
    				params.getBytes(paramEncoding), "grammarCallBack", this);
    		DebugLog.LogD( "QISRBuildGrammar leave: "+ret );
    	} catch (UnsupportedEncodingException e) {
    		DebugLog.LogE(e);
    		return ErrorCode.ERROR_INVALID_PARAM;
    	}
    	return ret;
    }

    /**
     * buildGrammar回调
     * 
     * @param errorCode
     * @param grammarID
     * @return
     */
    int grammarCallBack(final int errorCode, final char[] grammarID) {
    	DebugLog.LogD( "grammarCallBack begin, errorCode: "+errorCode );
    	PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
        if (mGrammarListener != null) {
            if (errorCode != 0) {
                mGrammarListener.onBuildFinish("", (errorCode == 0) ? null
                        : new SpeechError(errorCode));
            } else {
                mGrammarListener.onBuildFinish(String.valueOf(grammarID), null);
            }
        }

        DebugLog.LogD( "grammarCallBack begin, leave" );
        return ErrorCode.SUCCESS;
    }

    /**
     * 更新云端或本地词典
     * 
     * @param lexiconName
     * @param lexiconContent
     * @param lexiconListener
     * @return
     */
    public int upDateLexicon(final String lexiconName,
            final String lexiconContent, final LexiconListener lexiconListener,
            final HashParam mSessionParams) {
        mLexiconListener = lexiconListener;
        mLexiconName = lexiconName;
        // 参数增加默认编码格式
        mSessionParams.putParam(SpeechConstant.TEXT_ENCODING, DataUtil.UTF8, false);
        //文本编码格式
    	String textEncoding = mSessionParams.getString(SpeechConstant.TEXT_ENCODING, DataUtil.UTF8);
    	//参数编码格式
    	String paramEncoding = mSessionParams.getString(MscKeys.KEY_PTE, DataUtil.UTF8);
        
        String grmUpdateString = mSessionParams.toString();
        int ret = ErrorCode.MSP_ERROR_FAIL;
        try {
        	byte[] tempLexiconContent = lexiconContent.getBytes(textEncoding);
        	DebugLog.LogD( "QISRUpdateLexicon begin, name: "+lexiconName+"content: "+lexiconContent );
        	PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
            ret = MSC.QISRUpdateLexicon(lexiconName.getBytes(paramEncoding),
            		tempLexiconContent,
            		tempLexiconContent.length,
                    grmUpdateString.getBytes(paramEncoding), "lexiconCallBack", this);
            DebugLog.LogD( "QISRUpdateLexicon leave: "+ret );
        } catch (UnsupportedEncodingException e) {
            DebugLog.LogE(e);
            return ErrorCode.ERROR_INVALID_PARAM;
        }
        return ret;
    }

    /**
     * 更新云端或本地词典 回调
     * 
     * @param errorCode
     * @param grammarID
     * @return
     */
    int lexiconCallBack(final int errorCode, final char[] grammarID) {
    	DebugLog.LogD( "lexiconCallBack begin, errorCode: "+errorCode );
    	PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
        if (mLexiconListener != null) {
            if (errorCode != 0) {
                mLexiconListener.onLexiconUpdated(mLexiconName, (errorCode == 0) ? null
                        : new SpeechError(errorCode));
            } else {
                mLexiconListener.onLexiconUpdated(mLexiconName, null);
            }
        }
        
        DebugLog.LogD( "lexiconCallBack leave" );
        return ErrorCode.SUCCESS;
    }

    /**
     * 上传、下载流量信息获取
     * 
     * @param tag
     * @return
     */
    public static int getIntByTag(final String tag) {
        try {
            String val = getStringByTag(tag);
            if (TextUtils.isEmpty(val)) {
                return 0;
            }
            return Integer.parseInt(val);
        } catch (Exception e) {
        }
        return 0;
    }

    /**
     * 根据tag获取msc相关的值
     * 
     * @param tag
     * @return
     */
    public static String getStringByTag(final String tag) {
        int code = 0;
        try {
            MSCSessionInfo info = new MSCSessionInfo();
            code = MSC.QISRGetParam(null, tag.getBytes(), info);
            if (code == 0) {
                return new String(info.buffer).trim();
            } else {
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * if engine has result,return true
     * 
     * @return
     */
    synchronized public boolean hasResult() {
        return mSessionOut.sesstatus == 0;
    }

    public byte[] getResultData() {
        return mResult;
    }

    public ResultStatus getStatus() throws SpeechError {

        long d = SystemClock.elapsedRealtime();
        mResult = MSC.QISRGetResult(mClientID, mSessionOut);
        DebugLog.LogI("QISRGetResult leavel:" + (null != mResult) + " time:"
                + (SystemClock.elapsedRealtime() - d));

        int error = mSessionOut.errorcode;
        if (error == 0) {
            int resultStatus = mSessionOut.rsltstatus;
            switch (resultStatus) {
            case 0:
                DebugLog.LogD("ResultStatus: hasResult" + resultStatus);
                return ResultStatus.hasResult;
            case 2:
                DebugLog.LogI("ResultStatus: noResult" + resultStatus);
                return ResultStatus.noResult;
            case 5:
                DebugLog.LogD("ResultStatus: resultOver" + resultStatus);
                return ResultStatus.resultOver;
            }
            return ResultStatus.noResult;
        }
        
        DebugLog.LogE("Result: error errorcode is " + error);
        throw new SpeechError(error);
    }

    /**
     * 获得服务端ID
     */
    protected String getSessionID() {
        if (null == mSessionID) {
            mSessionID = getStringValue(TAG_SID);
        }
        return mSessionID;
    }
    
    /**
     * 获得音频url
     */
    protected String getAudioUrl() {
    	return  getStringValue(TAG_AUDIO_URL);
    }
}
